const { isEmpty, replace } = require('lodash');

const required = () => {
  return (value) => {
    return isEmpty(value) ? 'This field is required' : true;
  };
};

const sanitize = value => replace(value, ' ', '');

module.exports = (plop) => {
  plop.setGenerator('page', {
    description: 'Generate a Page',
    prompts: [
      {
        type: 'input',
        name: 'name',
        message: 'Enter name of Page:',
        validate: required(),
      },
      {
        type: 'input',
        name: 'element',
        message: 'What is the main HTML element wrapper? Default:',
        default: 'div',
        filter: sanitize,
      },
    ],
    actions: [
      {
        type: 'add',
        path: 'src/pages/{{pascalCase name}}/index.tsx',
        templateFile: 'plop-templates/pages_components/Page_Component/Page_index.tsx.hbs',
      },
      {
        type: 'add',
        path: 'src/pages/{{pascalCase name}}/{{pascalCase name}}.scss',
        templateFile: 'plop-templates/pages_components/Page_Component/Page_Component.scss.hbs'
      },
      {
        type: 'add',
        path: 'src/pages/index.ts',
        templateFile: 'plop-templates/pages_components/index.ts.hbs',
        skipIfExists: true,
      },
      {
        type: 'append',
        path: 'src/pages/index.ts',
        pattern: '/* INDEX_EXPORTS */',
        template: "export * from './{{pascalCase name}}';",
      },
      {
        type: 'append',
        path: 'src/index.tsx',
        pattern: '/* PAGE_IMPORTS */',
        template: '\t{{pascalCase name}},',
      },
      {
        type: 'append',
        path: 'src/index.tsx',
        pattern: '<Routes>',
        template: "\t\t\t\t\t<Route path='/{{dashCase name}}' element={<{{pascalCase name}} />} />",
      },
    ],
  });

  plop.setGenerator('component', {
    description: 'Generate a Component',
    prompts: [
      {
        type: 'input',
        name: 'name',
        message: 'Enter name of Component:',
        validate: required(),
      },
      {
        type: 'input',
        name: 'element',
        message: 'What is the main HTML element wrapper? Default:',
        default: 'div',
        filter: sanitize,
      },
    ],
    actions: [
      {
        type: 'add',
        path: 'src/components/{{pascalCase name}}/index.tsx',
        templateFile: 'plop-templates/pages_components/Page_Component/Component_index.tsx.hbs',
      },
      {
        type: 'add',
        path: 'src/components/{{pascalCase name}}/{{pascalCase name}}.scss',
        templateFile: 'plop-templates/pages_components/Page_Component/Page_Component.scss.hbs'
      },
      {
        type: 'add',
        path: 'src/components/index.ts',
        templateFile: 'plop-templates/pages_components/index.ts.hbs',
        skipIfExists: true,
      },
      {
        type: 'append',
        path: 'src/components/index.ts',
        pattern: '/* INDEX_EXPORTS */',
        template: "export * from './{{pascalCase name}}';",
      },
    ],
  });

  plop.setGenerator('redux', {
    description: 'Generate a Redux Structure',
    prompts: [
      {
        type: 'input',
        name: 'model',
        message: 'Enter name of Model',
        validate: required(),
      },
      {
        type: 'input',
        name: 'page',
        message: 'Name of page to add Redux Store imports',
        default: 'app',
      },
    ],
    actions: [
      {
        type: 'append',
        path: 'src/pages/{{pascalCase page}}/index.tsx',
        pattern: "import { useAppDispatch, useAppSelector } from '../../hooks';",
        template: '\nimport {\n' +
          '\tgetAll{{pascalCase model}}s,\n' +
          '\tgetOne{{pascalCase model}},\n' +
          "} from '../../controllers';\n\n" +
          'import {\n' +
          '\tselectAll{{pascalCase model}}s,\n' +
          '\tselectAll{{pascalCase model}}sError,\n' +
          '\tselectAll{{pascalCase model}}sStatus,\n' +
          '\tselectOne{{pascalCase model}},\n' +
          '\tselectOne{{pascalCase model}}Error,\n' +
          '\tselectOne{{pascalCase model}}Status,\n' +
          "} from '../../store';",
        abortOnFail: true,
      },
      {
        type: 'append',
        path: 'src/pages/{{pascalCase page}}/index.tsx',
        pattern: 'const dispatch = useAppDispatch();',
        template: '\tconst all{{pascalCase model}}s = useAppSelector(selectAll{{pascalCase model}}s);\n' +
          '\tconst all{{pascalCase model}}sStatus = useAppSelector(selectAll{{pascalCase model}}sStatus);\n' +
          '\tconst all{{pascalCase model}}sError = useAppSelector(selectAll{{pascalCase model}}sError);\n' +
          '\tconst { type, id, attributes } = useAppSelector(selectOne{{pascalCase model}});\n' +
          '\tconst one{{pascalCase model}}Status = useAppSelector(selectOne{{pascalCase model}}Status);\n' +
          '\tconst one{{pascalCase model}}Error = useAppSelector(selectOne{{pascalCase model}}Error);\n\n'+
          '\tuseEffect(() => {\n' +
          '\t\tdispatch(getAll{{pascalCase model}}s());\n' +
          '\t\tdispatch(getOne{{pascalCase model}}(1));\n' +
          '\t}, []);',
        abortOnFail: true,
      },
      {
        type: 'add',
        path: 'src/models/{{pascalCase model}}.ts',
        templateFile: 'plop-templates/models/Model.ts.hbs',
      },
      {
        type: 'add',
        path: 'src/models/index.ts',
        templateFile: 'plop-templates/models/index.ts.hbs',
        skipIfExists: true,
      },
      {
        type: 'append',
        path: 'src/models/index.ts',
        pattern: '/* MODEL_EXPORTS */',
        template: "export * from './{{pascalCase model}}';",
      },
      {
        type: 'add',
        path: 'src/controllers/{{camelCase model}}Controller.ts',
        templateFile: 'plop-templates/controllers/modelController.ts.hbs',
      },
      {
        type: 'add',
        path: 'src/controllers/index.ts',
        templateFile: 'plop-templates/controllers/index.ts.hbs',
        skipIfExists: true,
      },
      {
        type: 'append',
        path: 'src/controllers/index.ts',
        pattern: '/* CONTROLLER_EXPORTS */',
        template: "export * from './{{camelCase model}}Controller';",
      },
      {
        type: 'add',
        path: 'src/requests/{{camelCase model}}Request.ts',
        templateFile: 'plop-templates/requests/modelRequest.ts.hbs',
      },
      {
        type: 'append',
        path: 'src/requests/index.ts',
        pattern: '/* REQUEST_EXPORTS */',
        template: "export * from './{{camelCase model}}Request';",
      },
      {
        type: 'add',
        path: 'src/store/{{pascalCase model}}/{{camelCase model}}Slice.ts',
        templateFile: 'plop-templates/store/Model/modelSlice.ts.hbs',
      },
      {
        type: 'add',
        path: 'src/store/{{pascalCase model}}/index.ts',
        templateFile: 'plop-templates/store/Model/index.ts.hbs',
      },
      {
        type: 'append',
        path: 'src/store/index.ts',
        pattern: "import { Action, configureStore, ThunkAction } from '@reduxjs/toolkit';",
        template: "import { {{pascalCase model}}Reducer } from './{{pascalCase model}}';",
      },
      {
        type: 'append',
        path: 'src/store/index.ts',
        pattern: 'reducer: {',
        template: '\t\t{{camelCase model}}: {{pascalCase model}}Reducer,'
      },
      {
        type: 'append',
        path: 'src/store/index.ts',
        pattern: '/* STORE_EXPORTS */',
        template: "export * from './{{pascalCase model}}';",
      },
    ],
  });

  plop.setGenerator('env', {
    description: 'Generate examples',
    actions: [
      {
        type: 'add',
        path: '.env',
        templateFile: 'plop-templates/.env.hbs',
        skipIfExists: true,
      },
    ],
  });
};
